/**************************************************************
 * SPRITE Demo
 *
 * Copyright 2013 William Henning
 *
 * visit http://Mikronauts.com/robot-zoo/sprite/
 *
 * for more information.
 *
 **************************************************************/

#include <stdio.h>
#include <pigpio.h>

#define	LMOTOR	4	// LEFT servo   
#define	RMOTOR	17	// RIGHT servo  
#define	PSONAR	18	// PAN servo	

#define SONAR	25 		// Ultrasonic Sensor

volatile uint32_t before;  // used for sonar distance

int last_range = 0;	     // last sonar reading

void servo(int s, int t) {
  // only accept valid servo settings
  if ((t<500)||(t>2500)) return; 
  if (s==PSONAR)
     printf("%4d: "); // for sweep, so we see pan angle for range
  gpioServo(s,t);
}

void delay(int ms) {  // delay in miliseconds
    gpioDelay(1000*ms); 
}

void ping(void) {     // send out an ultrasonic 'ping'

   before = 0xffffffff; // set for guard variable

   gpioSetMode(SONAR, PI_OUTPUT);

   // trigger a sonar pulse
   
   gpioWrite(SONAR, PI_OFF);
   gpioDelay(5);
   gpioWrite(SONAR, PI_ON);
   gpioDelay(10); 	
   gpioWrite(SONAR, PI_OFF);
   gpioDelay(5);

   gpioSetMode(SONAR, PI_INPUT);

   before = gpioTick(); // get tick right after sending pulse
   
}

void sonar(int t) {
   servo(PSONAR,t);
   ping();
}

// range - callback function for measuring ping response

void range(int gpio, int level, uint32_t tick) {

   static uint32_t startTick, endTick;
   
   uint32_t diffTick;

   if (tick>before) { // make sure we don't measure trigger pulse

      if (level == PI_ON) { // start counting on rising edge
         startTick = tick;
      } else 
      
      if (level == PI_OFF) { // stop counting on falling edge

         endTick = tick;
         diffTick = (endTick - startTick)/58;

         last_range = diffTick;

         if (diffTick < 600)
            printf("%u\n", diffTick);
         else {
            printf("OUT OF RANGE"); // for seeedstudio sensor
            last_range = 0;
         }
      }

   }

}

// sweeping sonar test

void sweep() {

   int i;

   servo(PSONAR,500);
   delay(300);

     for(i=500;i<=2500;i+=125) {        
         sonar(i);
         delay(150);    
      }
    
      for(i=2500;i>=500;i-=125) {        
         sonar(i);
         delay(150);    
      }

   servo(PSONAR,1500);
   delay(300);
   
}

void sleep(int t) {
  gpioSleep(PI_TIME_RELATIVE, t, 0);
}

// the main demonstration program

int main(int argc, char *argv[]) {
  
   int i, n;
   char buff[128], ch;          

   if (gpioInitialise()<0) return 1;
   
   gpioSetMode(SONAR, PI_INPUT);

   // register callback on change of sonar pin
   gpioSetAlertFunc(SONAR, range); 

   servo(LMOTOR,1500);
   servo(RMOTOR,1500);
   servo(PSONAR,1500);
    
   sleep(2);    

   while (buff[0]!='q') {
   
      // show command menu
      puts("l{n}  Spin Left");
      puts("r{n}  Spin Right");
      puts("f{n}  Go Forward");
      puts("b{n}  Go Back");
      puts("s     Sonar Sweep");
      puts("w     Ramdom walk, avoids obstacles");
      puts("q     Exit");
      puts("\n{n} is an optional digit for number of seconds");
   
      gets(&buff[0]);
      ch = buff[0];
      
      n = buff[1];
      
      if ((n >= '0')&&(n<='9'))  // handle single digit argument
         n -= 47;
      else
         n = 1; // if no argument, assume 1
      
      switch (ch) {
      
         case 'f': // go forwards slowly
            servo(LMOTOR, 1100);
            servo(RMOTOR, 3000-1100);
            sleep(1*n);
            break;
                        
         case 'b': // go backwards slowly
            servo(LMOTOR, 1900);
            servo(RMOTOR, 3000-1900);
            sleep(1*n);
            break;   

         case 'l': // slow turn left
            servo(LMOTOR, 2000);
            servo(RMOTOR, 2000);
            delay(500*n);
            break;            
            
         case 'r': // slow turn right
            servo(LMOTOR, 1000);
            servo(RMOTOR, 1000);
            delay(500*n);
            break;
            
         case 's': // sweep the sonar with the pan servo
            sweep();
            break;
         
         case 'w': // random walk with obstacle avoidance
         
            ping();	// prime the last_range variable
            sleep(1);

            while (1) {
            
	          // while path ahead is clear for at least 30cm
               while (last_range > 30) {
                  ping();
                  delay(100);
                  servo(LMOTOR, 500);
                  servo(RMOTOR, 3000-500);
               }

               puts("Obstacle detected");            

               // look for at least 50cm of open space
               while (last_range < 50) {
                  ping();
                  delay(100);
                  servo(LMOTOR, 500);
                  servo(RMOTOR, 500);
               }            
            }

            servo(LMOTOR, 1500);
            servo(RMOTOR, 1500);
            sleep(1);
            break;

         case 'q':
         case 'Q': 
         
            break;

         default:
         
            puts("Huh?");
      }     

      servo(LMOTOR,1500);
      servo(RMOTOR,1500);
      servo(PSONAR,1500);

   }

   sleep(1);
    
   puts("Bye now!");  

   gpioTerminate();

   return 0;

}
